/**
 * <pre>
 * Listens to HELP_COMMAND and displays notifications.
 * Provides interactive tutorial for first time users.
 * </pre>
 *
 * @constructor
 * @param {mindmaps.EventBus} eventBus
 * @param {mindmaps.commandRegistry} commandRegistry
 */
mindmaps.HelpController = function (eventBus, commandRegistry) {

    /**
     * Prepare tutorial guiders.
     */
    function setupInteractiveMode() {
        if (isTutorialDone()) {
            console.debug("passer le tutoriel");
            return;
        }

        var notifications = [];
        var interactiveMode = true;

        // start tutorial after a short delay
        eventBus.once(mindmaps.Event.DOCUMENT_OPENED, function () {
            setTimeout(start, 1000);
        });

        function closeAllNotifications() {
            notifications.forEach(function (n) {
                n.close();
            });
        }

        var helpMain, helpRoot;

        function start() {
            helpMain = new mindmaps.Notification(
                "#toolbar",
                {
                    position: "bottomMiddle",
                    maxWidth: 550,
                    title : "Bienvenue dans MindMaps",
            content : "Les bulles vous guideront dans l'application."
                + "<br>Si vous souhaitez passer ce Pas à Pas <a class='skip-tutorial link'>Cliquez ICI<a/>."
                });
            notifications.push(helpMain);
            helpMain.$().find("a.skip-tutorial").click(function () {
                interactiveMode = false;
                closeAllNotifications();
                tutorialDone();
            });
            setTimeout(theRoot, 2000);
        }

        function theRoot() {
            if (isTutorialDone())
                return;

            helpRoot = new mindmaps.Notification(
                ".node-caption.root",
                {
                    position: "bottomMiddle",
                    closeButton: true,
                    maxWidth: 350,
                    title : "Commencez ici par écrire votre idée principale",
					content : "Ce sera le sujet principal de votre Carte Mentale."+"<br>Double cliquez sur l'étiquette pour changer son contenu."
                });
            notifications.push(helpRoot);

            eventBus.once(mindmaps.Event.NODE_TEXT_CAPTION_CHANGED, function () {
                helpRoot.close();
                setTimeout(theNub, 900);
            });
        }

        function theNub() {
            if (isTutorialDone())
                return;

            var helpNub = new mindmaps.Notification(
                ".node-caption.root",
                {
                    position: "bottomMiddle",
                    closeButton: true,
                    maxWidth: 350,
                    padding: 20,
                    title : "Ajoutez de Nouvelles Idées",
					content : "Maintenant, il est temps de construire votre Carte Mentale.<br/> Placez le curseur de la souris sur une étiquette, cliquez puis déplacez"
					+ " les <span style='color:red'>bouton rouge</span> hors de la racine. Cela crée une branche."
                });
            notifications.push(helpNub);
            eventBus.once(mindmaps.Event.NODE_CREATED, function () {
                helpMain.close();
                helpNub.close();
                setTimeout(newNode, 900);
            });
        }

        function newNode() {
            if (isTutorialDone())
                return;

            var helpNewNode = new mindmaps.Notification(
                ".node-container.root > .node-container:first",
                {
                    position: "bottomMiddle",
                    closeButton: true,
                    maxWidth: 350,
                    title : "Votre Première Branche",
					content : "Bien! C'est facile, non? Le bouton rouge est l'outil le plus important. Maintenant, vous pouvez déplacez vos idées"
					+ " autour ou changer leur contenu en double cliquant dessus."
                });
            notifications.push(helpNewNode);
            setTimeout(inspector, 2000);

            eventBus.once(mindmaps.Event.NODE_MOVED, function () {
                helpNewNode.close();
                setTimeout(navigate, 0);
                setTimeout(toolbar, 15000);
                setTimeout(menu, 10000);
                setTimeout(tutorialDone, 20000);
            });
        }

        function navigate() {
            if (isTutorialDone())
                return;

            var helpNavigate = new mindmaps.Notification(
                ".float-panel:has(#navigator)",
                {
                    position: "bottomRight",
                    closeButton: true,
                    maxWidth: 350,
                    expires: 10000,
                    title : "Navigation",
					content : "Vous pouvez cliquer et déplacer le fond de la carte pour vous déplacer autour. Utilisez la molette de la souris ou le curseur pour zommer/dézoomer."
                });
            notifications.push(helpNavigate);
        }

        function inspector() {
            if (isTutorialDone())
                return;

            var helpInspector = new mindmaps.Notification(
                "#inspector",
                {
                    position: "leftBottom",
                    closeButton: true,
                    maxWidth: 350,
                    padding: 20,
                    title : "Vous n'aimez pas les couleurs ?",
					content : "Utilisez les contrôles pour modifier la couleur des idées. "
					+ "<br>Cliquez sur l'icône dans l'angle en haut à droite pour fermer la fenêtre."
                });
            notifications.push(helpInspector);
        }

        function toolbar() {
            if (isTutorialDone())
                return;

            var helpToolbar = new mindmaps.Notification(
                "#toolbar .buttons-left",
                {
                    position: "bottomLeft",
                    closeButton: true,
                    maxWidth: 350,
                    padding: 20,
                    title : "La Barre d'outils",
					content : "Ces boutons font ce qu'ils indiquent. Vous pouvez les utiliser ou vous servir des raccourcis clavier. "
					+ "Survolez les  boutons pour voir les combinaisons de touches."
                });
            notifications.push(helpToolbar);
        }

        function menu() {
            if (isTutorialDone())
                return;

            var helpMenu = new mindmaps.Notification(
                "#toolbar .buttons-right",
                {
                    position: "leftTop",
                    closeButton: true,
                    maxWidth: 350,
                    title : "Enregistrez votre travail",
					content : "Le bouton à droite ouvre un menu qui vous pemet d'enregistrer votre Carte "
					+ "ou commencer un nouveau projet si vous préférez."
                });
            notifications.push(helpMenu);
        }

        function isTutorialDone() {
            return mindmaps.LocalStorage.get("mindmaps.tutorial.done") == 1;
        }

        function tutorialDone() {
            mindmaps.LocalStorage.put("mindmaps.tutorial.done", 1);
        }

    }

    /**
     * Prepares notfications to show for help command.
     */
    function setupHelpButton() {
        var command = commandRegistry.get(mindmaps.HelpCommand);
        command.setHandler(showHelp);

        var notifications = [];

        function showHelp() {
            // true if atleast one notifications is still on screen
            var displaying = notifications.some(function (noti) {
                return noti.isVisible();
            });

            // hide notifications if visible
            if (displaying) {
                notifications.forEach(function (noti) {
                    noti.close();
                });
                notifications.length = 0;
                return;
            }

            // show notifications
            var helpRoot = new mindmaps.Notification(
                ".node-caption.root",
                {
                    position: "bottomLeft",
                    closeButton: true,
                    maxWidth: 350,
                    title : "L'idée principale",
					content : "Double cliquez sur une étiquette pour modifier le texte. Placez la souris sur une étiquette "
					+ "et déplacez le bouton rouge pour créer une nouvelle étiquette."
                });

            var helpNavigator = new mindmaps.Notification(
                "#navigator",
                {
                    position: "leftTop",
                    closeButton: true,
                    maxWidth: 350,
                    padding: 20,
                    title : "L'Aperçu",
					content : "Utilisez ce panneau pour avoir un aperçu de votre Carte. "
					+ "Vous pouvez naviguez sur la Carte en déplaçant le rectangle rouge et zoomer grâce aux loupes ou au curseur."
                });

            var helpInspector = new mindmaps.Notification(
                "#inspector",
                {
                    position: "leftTop",
                    closeButton: true,
                    maxWidth: 350,
                    padding: 20,
                    title : "L'Apparence",
					content : "Utilisez les contrôles pour modifier l'apparence des étiquettes. "
					+ "Cliquez sur l'icône en haut à droite pour fermer la fenêtre."
                });

            var helpToolbar = new mindmaps.Notification(
                "#toolbar .buttons-left",
                {
                    position: "bottomLeft",
                    closeButton: true,
                    maxWidth: 350,
                    title : "La Barre d'outils",
					content : "Vous pouvez utiliser les boutons ou vous servir des raccourcis clavier. "
					+ "Survolez les boutons pour voir les combinaisons de touches."
                });

            notifications.push(helpRoot, helpNavigator, helpInspector,
                helpToolbar);
        }
    }

    setupInteractiveMode();
    setupHelpButton();
};
